@@ -1,21 +1,8 @@
 /*
- *  Copyright (c) 2005 Adrian Page <adrian@pagenet.plus.com>
- *  Copyright (c) 2010 Cyrille Berger <cberger@cberger.net>
+ *  SPDX-FileCopyrightText: 2005 Adrian Page <adrian@pagenet.plus.com>
+ *  SPDX-FileCopyrightText: 2010 Cyrille Berger <cberger@cberger.net>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "exr_converter.h"
@@ -24,6 +11,8 @@
 
 #include <ImfAttribute.h>
 #include <ImfChannelList.h>
+#include <ImfFrameBuffer.h>
+#include <ImfHeader.h>
 #include <ImfInputFile.h>
 #include <ImfOutputFile.h>
 
@@ -333,7 +322,7 @@ void EXRConverter::Private::unmultiplyAlpha(typename WrapperType::pixel_type *pi
 
         /**
          * Division by a tiny alpha may result in an overflow of half
-         * value. That is why we use safe iterational approach.
+         * value. That is why we use safe iterative approach.
          */
         while (1) {
             dstPixel.setUnmultiplied(srcPixel.pixel, newAlpha);
@@ -439,8 +428,8 @@ void EXRConverter::Private::decodeData1(Imf::InputFile& file, ExrPaintLayerInfo&
 
     QVector<pixel_type> pixels(width * height);
 
-    Q_ASSERT(info.channelMap.contains("G"));
-    dbgFile << "G -> " << info.channelMap["G"];
+    Q_ASSERT(info.channelMap.contains("Y"));
+    dbgFile << "Gray -> " << info.channelMap["Y"];
 
     bool hasAlpha = info.channelMap.contains("A");
     dbgFile << "Has Alpha:" << hasAlpha;
@@ -448,10 +437,9 @@ void EXRConverter::Private::decodeData1(Imf::InputFile& file, ExrPaintLayerInfo&
 
     Imf::FrameBuffer frameBuffer;
     pixel_type* frameBufferData = (pixels.data()) - xstart - ystart * width;
-    frameBuffer.insert(info.channelMap["G"].toLatin1().constData(),
-            Imf::Slice(ptype, (char *) &frameBufferData->gray,
-                       sizeof(pixel_type) * 1,
-                       sizeof(pixel_type) * width));
+    frameBuffer.insert(
+        info.channelMap["Y"].toLatin1().constData(),
+        Imf::Slice(ptype, (char *)&frameBufferData->gray, sizeof(pixel_type) * 1, sizeof(pixel_type) * width));
 
     if (hasAlpha) {
         frameBuffer.insert(info.channelMap["A"].toLatin1().constData(),
@@ -467,8 +455,7 @@ void EXRConverter::Private::decodeData1(Imf::InputFile& file, ExrPaintLayerInfo&
 
     QRect paintRegion(xstart, ystart, width, height);
     KisSequentialIterator it(layer->paintDevice(), paintRegion);
-    do {
-
+    while (it.nextPixel()) {
         if (hasAlpha) {
             unmultiplyAlpha<GrayPixelWrapper<_T_> >(srcPtr);
         }
@@ -479,7 +466,7 @@ void EXRConverter::Private::decodeData1(Imf::InputFile& file, ExrPaintLayerInfo&
         dstPtr->alpha = hasAlpha ? srcPtr->alpha : channel_type(1.0);
 
         ++srcPtr;
-    } while (it.nextPixel());
+    } ;
 }
 
 bool recCheckGroup(const ExrGroupLayerInfo& group, QStringList list, int idx1, int idx2)
@@ -547,7 +534,7 @@ bool EXRConverter::Private::checkExtraLayersInfoConsistent(const QDomDocument &d
     bool result = (extraInfoLayers == exrLayerNames);
 
     if (!result) {
-        dbgKrita << "WARINING: Krita EXR extra layers info is inconsistent!";
+        dbgKrita << "WARNING: Krita EXR extra layers info is inconsistent!";
         dbgKrita << ppVar(extraInfoLayers.size()) << ppVar(exrLayerNames.size());
 
         std::set<std::string>::const_iterator it1 = extraInfoLayers.begin();
@@ -567,7 +554,7 @@ bool EXRConverter::Private::checkExtraLayersInfoConsistent(const QDomDocument &d
 KisImportExportErrorCode EXRConverter::decode(const QString &filename)
 {
     try {
-        Imf::InputFile file(QFile::encodeName(filename));
+        Imf::InputFile file(filename.toUtf8());
 
         Imath::Box2i dw = file.header().dataWindow();
         Imath::Box2i displayWindow = file.header().displayWindow();
@@ -611,11 +598,29 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
         bool topLevelRGBFound = false;
         info.name = HDR_LAYER;
 
-        QStringList topLevelChannelNames = QStringList() << "A" << "R" << "G" << "B"
-                                                         << ".A" << ".R" << ".G" << ".B"
-                                                         << "A." << "R." << "G." << "B."
-                                                         << "A." << "R." << "G." << "B."
-                                                         << ".alpha" << ".red" << ".green" << ".blue";
+        QStringList topLevelChannelNames = QStringList() << "A"
+                                                         << "R"
+                                                         << "G"
+                                                         << "B"
+                                                         << ".A"
+                                                         << ".R"
+                                                         << ".G"
+                                                         << ".B"
+                                                         << "A."
+                                                         << "R."
+                                                         << "G."
+                                                         << "B."
+                                                         << "A."
+                                                         << "R."
+                                                         << "G."
+                                                         << "B."
+                                                         << ".alpha"
+                                                         << ".red"
+                                                         << ".green"
+                                                         << ".blue"
+                                                         << "Y"
+                                                         << ".Y"
+                                                         << "Y.";
 
         for (Imf::ChannelList::ConstIterator i = channels.begin(); i != channels.end(); ++i) {
             const Imf::Channel &channel = i.channel();
@@ -699,11 +704,11 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
             if (info.channelMap.size() == 1) {
                 modelId = GrayAColorModelID.id();
                 QString key = info.channelMap.begin().key();
-                if (key != "G") {
-                    info.remappedChannels.push_back(ExrPaintLayerInfo::Remap(key, "G"));
+                if (key != "Y") {
+                    info.remappedChannels.push_back(ExrPaintLayerInfo::Remap(key, "Y"));
                     QString channel =  info.channelMap.begin().value();
                     info.channelMap.clear();
-                    info.channelMap["G"] = channel;
+                    info.channelMap["Y"] = channel;
                 }
             }
             else if (info.channelMap.size() == 2) {
@@ -715,19 +720,18 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
                 QString failingChannelKey;
 
                 for (; it != end; ++it) {
-                    if (it.key() != "G" && it.key() != "A") {
+                    // BUG: 461975
+                    if (it.key() != "A") {
                         failingChannelKey = it.key();
                         break;
                     }
                 }
 
-                info.remappedChannels.push_back(
-                            ExrPaintLayerInfo::Remap(failingChannelKey, "G"));
+                info.remappedChannels.push_back(ExrPaintLayerInfo::Remap(failingChannelKey, "Y"));
 
                 QString failingChannelValue = info.channelMap[failingChannelKey];
                 info.channelMap.remove(failingChannelKey);
-                info.channelMap["G"] = failingChannelValue;
-
+                info.channelMap["Y"] = failingChannelValue;
             }
             else if (info.channelMap.size() == 3 || info.channelMap.size() == 4) {
 
@@ -786,7 +790,7 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
         const KoColorSpace* colorSpace = kisTypeToColorSpace(RGBAColorModelID.id(), imageType);
 
         if (!colorSpace) return ImportExportCodes::FormatColorSpaceUnsupported;
-        dbgFile << "Colorspace: " << colorSpace->name();
+        dbgFile << "Color space: " << colorSpace->name();
 
         // Set the colorspace on all groups
         for (int i = 0; i < groups.size(); ++i) {
@@ -808,8 +812,12 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
         /**
          * EXR semi-transparent images are expected to be rendered on
          * black to ensure correctness of the light model
+         *
+         * NOTE: We cannot do that automatically, because the EXR may be imported
+         * into the image as a layer, in which case the default color will create
+         * major issues. See https://bugs.kde.org/show_bug.cgi?id=427720
          */
-        d->image->setDefaultProjectionColor(KoColor(Qt::black, colorSpace));
+        //d->image->setDefaultProjectionColor(KoColor(Qt::black, colorSpace));
 
         // Create group layers
         for (int i = 0; i < groups.size(); ++i) {
@@ -885,8 +893,6 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
                 dbgFile << "No decoding " << info.name << " with " << info.channelMap.size() << " channels, and lack of a color space";
             }
         }
-        // Set projectionColor to opaque
-        d->image->setDefaultProjectionColor(KoColor(Qt::transparent, colorSpace));
 
         // After reading the image, notify the user about changed alpha.
         if (d->alphaWasModified) {
@@ -899,7 +905,7 @@ KisImportExportErrorCode EXRConverter::decode(const QString &filename)
                           "<br/><br/>"
                           "This will hardly make any visual difference just keep it in mind.");
             if (d->showNotifications) {
-                QMessageBox::information(0, i18nc("@title:window", "EXR image has been modified"), msg);
+                QMessageBox::information(qApp->activeWindow(), i18nc("@title:window", "EXR image has been modified"), msg);
             } else {
                 warnKrita << "WARNING:" << msg;
             }
@@ -1077,7 +1083,7 @@ KisPaintDeviceSP wrapLayerDevice(KisPaintDeviceSP device)
             cs->colorModelId() == GrayAColorModelID ?
                 GrayAColorModelID.id() : RGBAColorModelID.id(),
             Float16BitsColorDepthID.id());
-    } else if (cs->colorModelId() != GrayColorModelID &&
+    } else if (cs->colorModelId() != GrayAColorModelID &&
                cs->colorModelId() != RGBAColorModelID) {
         cs = KoColorSpaceRegistry::instance()->colorSpace(
             RGBAColorModelID.id(),
@@ -1115,20 +1121,40 @@ KisImportExportErrorCode EXRConverter::buildFile(const QString &filename, KisPai
     else if (info.layerDevice->colorSpace()->colorDepthId() == Float32BitsColorDepthID) {
         pixelType = Imf::FLOAT;
     }
-    header.channels().insert("R", Imf::Channel(pixelType));
-    header.channels().insert("G", Imf::Channel(pixelType));
-    header.channels().insert("B", Imf::Channel(pixelType));
-    header.channels().insert("A", Imf::Channel(pixelType));
 
-    info.channels.push_back("R");
-    info.channels.push_back("G");
-    info.channels.push_back("B");
-    info.channels.push_back("A");
     info.pixelType = pixelType;
 
+    if (info.layerDevice->colorSpace()->colorModelId() == RGBAColorModelID) {
+        header.channels().insert("R", Imf::Channel(pixelType));
+        header.channels().insert("G", Imf::Channel(pixelType));
+        header.channels().insert("B", Imf::Channel(pixelType));
+        header.channels().insert("A", Imf::Channel(pixelType));
+
+        info.channels.push_back("R");
+        info.channels.push_back("G");
+        info.channels.push_back("B");
+        info.channels.push_back("A");
+    } else if (info.layerDevice->colorSpace()->colorModelId() == GrayAColorModelID) {
+        header.channels().insert("Y", Imf::Channel(pixelType));
+        header.channels().insert("A", Imf::Channel(pixelType));
+
+        info.channels.push_back("Y");
+        info.channels.push_back("A");
+    } else if (info.layerDevice->colorSpace()->colorModelId() == XYZAColorModelID) {
+        header.channels().insert("X", Imf::Channel(pixelType));
+        header.channels().insert("Y", Imf::Channel(pixelType));
+        header.channels().insert("Z", Imf::Channel(pixelType));
+        header.channels().insert("A", Imf::Channel(pixelType));
+
+        info.channels.push_back("X");
+        info.channels.push_back("Y");
+        info.channels.push_back("Z");
+        info.channels.push_back("A");
+    }
+
     // Open file for writing
     try {
-        Imf::OutputFile file(QFile::encodeName(filename), header);
+        Imf::OutputFile file(filename.toUtf8(), header);
 
         QList<ExrPaintLayerSaveInfo> informationObjects;
         informationObjects.push_back(info);
@@ -1137,7 +1163,7 @@ KisImportExportErrorCode EXRConverter::buildFile(const QString &filename, KisPai
 
     } catch(std::exception &e) {
         dbgFile << "Exception while writing to exr file: " << e.what();
-        if (!KisImportExportAdditionalChecks::isFileWritable(QFile::encodeName(filename))) {
+        if (!KisImportExportAdditionalChecks::isFileWritable(filename)) {
             return ImportExportCodes::NoAccessToWrite;
         }
         return ImportExportCodes::ErrorWhileWriting;
@@ -1233,32 +1259,27 @@ void EXRConverter::Private::recBuildPaintLayerSaveInfo(QList<ExrPaintLayerSaveIn
             }
             else {
 
-                if (paintLayer->colorSpace()->colorModelId() == RGBAColorModelID) {
+                if (info.layerDevice->colorSpace()->colorModelId() == RGBAColorModelID) {
                     info.channels.push_back(info.name + remap(current2original, "R"));
                     info.channels.push_back(info.name + remap(current2original, "G"));
                     info.channels.push_back(info.name + remap(current2original, "B"));
                     info.channels.push_back(info.name + remap(current2original, "A"));
                 }
-                else if (paintLayer->colorSpace()->colorModelId() == GrayAColorModelID) {
-                    info.channels.push_back(info.name + remap(current2original, "G"));
+                else if (info.layerDevice->colorSpace()->colorModelId() == GrayAColorModelID) {
+                    info.channels.push_back(info.name + remap(current2original, "Y"));
                     info.channels.push_back(info.name + remap(current2original, "A"));
-                }
-                else if (paintLayer->colorSpace()->colorModelId() == GrayColorModelID) {
-                    info.channels.push_back(info.name + remap(current2original, "G"));
-                }
-                else if (paintLayer->colorSpace()->colorModelId() == XYZAColorModelID) {
+                } else if (info.layerDevice->colorSpace()->colorModelId() == XYZAColorModelID) {
                     info.channels.push_back(info.name + remap(current2original, "X"));
                     info.channels.push_back(info.name + remap(current2original, "Y"));
                     info.channels.push_back(info.name + remap(current2original, "Z"));
                     info.channels.push_back(info.name + remap(current2original, "A"));
                 }
-
             }
 
-            if (paintLayer->colorSpace()->colorDepthId() == Float16BitsColorDepthID) {
+            if (info.layerDevice->colorSpace()->colorDepthId() == Float16BitsColorDepthID) {
                 info.pixelType = Imf::HALF;
             }
-            else if (paintLayer->colorSpace()->colorDepthId() == Float32BitsColorDepthID) {
+            else if (info.layerDevice->colorSpace()->colorDepthId() == Float32BitsColorDepthID) {
                 info.pixelType = Imf::FLOAT;
             }
             else {
@@ -1380,12 +1401,12 @@ KisImportExportErrorCode EXRConverter::buildFile(const QString &filename, KisGro
 
         // Open file for writing
         try {
-            Imf::OutputFile file(QFile::encodeName(filename), header);
+            Imf::OutputFile file(filename.toUtf8(), header);
             encodeData(file, informationObjects, width, height);
             return ImportExportCodes::OK;
         } catch(std::exception &e) {
             dbgFile << "Exception while writing to exr file: " << e.what();
-            if (!KisImportExportAdditionalChecks::isFileWritable(QFile::encodeName(filename))) {
+            if (!KisImportExportAdditionalChecks::isFileWritable(filename.toUtf8())) {
                 return ImportExportCodes::NoAccessToWrite;
             }
             return ImportExportCodes::ErrorWhileWriting;
